Package: struct
Type: Package
Title: Statistics in R Using Class-based Templates
Version: 1.25.0
Authors@R: c(
    person(
        c("Gavin","Rhys"),
          "Lloyd",
           role=c("aut","cre"),
           email="g.r.lloyd@bham.ac.uk"),
    person(
        c("Ralf","Johannes", "Maria"),
          "Weber",
           role=c("aut"),
           email="r.j.weber@bham.ac.uk")
           )
Description: Defines and includes a set of class-based templates for
        developing and implementing data processing and analysis
        workflows, with a strong emphasis on statistics and machine
        learning. The templates can be used and where needed extended
        to 'wrap' tools and methods from other packages into a common
        standardised structure to allow for effective and fast
        integration. Model objects can be combined into sequences, and
        sequences nested in iterators using overloaded operators to
        simplify and improve readability of the code. Ontology lookup
        has been integrated and implemented to provide standardised
        definitions for methods, inputs and outputs wrapped using the
        class-based templates.
License: GPL-3
Encoding: UTF-8
Collate: 'generics.R' 'ontology_term_class.R' 'struct_class.R'
        'parameter_class.R' 'chart_class.R' 'DatasetExperiment_class.R'
        'entity_class.R' 'enum_class.R' 'output_class.R'
        'model_class.R' 'example_objects.R' 'model_list_class.R'
        'metric_class.R' 'iterator_class.R' 'optimiser_class.R'
        'preprocess_class.R' 'resampler_class.R' 'struct-package.R'
        'struct_templates.R' 'zzz.R'
RoxygenNote: 7.3.3
Depends: R (>= 4.0)
Suggests: testthat, rstudioapi, rmarkdown, covr, BiocStyle, openxlsx,
        ggplot2, magick
VignetteBuilder: knitr
Imports: methods, datasets, graphics, stats, utils, knitr,
        SummarizedExperiment, S4Vectors, httr2, jsonlite
biocViews: WorkflowStep
Config/pak/sysreqs: libssl-dev zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:51:45 UTC
RemoteUrl: https://github.com/bioc/struct
RemoteRef: HEAD
RemoteSha: 1d098a078eb6bb726ab0d788d02d2b9c47df2918
NeedsCompilation: no
Packaged: 2026-05-30 09:44:36 UTC; root
Author: Gavin Rhys Lloyd [aut, cre],
  Ralf Johannes Maria Weber [aut]
Maintainer: Gavin Rhys Lloyd <g.r.lloyd@bham.ac.uk>
Built: R 4.6.0; ; 2026-05-30 14:22:30 UTC; unix
