# Generated by roxygen2: do not edit by hand

S3method(.DollarNames,DatasetExperiment)
S3method(.DollarNames,chart)
S3method(.DollarNames,iterator)
S3method(.DollarNames,metric)
S3method(.DollarNames,model)
S3method(.DollarNames,optimiser)
S3method(.DollarNames,preprocess)
S3method(.DollarNames,resampler)
S3method(.DollarNames,struct_class)
export("output_list<-")
export("output_value<-")
export("param_list<-")
export("param_obj<-")
export("param_value<-")
export(.struct_class)
export(DatasetExperiment)
export(as.DatasetExperiment)
export(as.SummarizedExperiment)
export(as.code)
export(as_data_frame)
export(calculate)
export(chart)
export(chart_names)
export(chart_plot)
export(citations)
export(entity)
export(enum)
export(example_chart)
export(example_iterator)
export(example_model)
export(get_description)
export(iris_DatasetExperiment)
export(is_output)
export(is_param)
export(iterator)
export(libraries)
export(max_length)
export(metric)
export(model)
export(model_apply)
export(model_predict)
export(model_reverse)
export(model_seq)
export(model_train)
export(models)
export(new_struct)
export(ontology)
export(ontology_list)
export(ontology_online)
export(ontology_term)
export(optimiser)
export(output_ids)
export(output_list)
export(output_name)
export(output_obj)
export(output_value)
export(param_ids)
export(param_list)
export(param_name)
export(param_obj)
export(param_value)
export(predicted)
export(predicted_name)
export(preprocess)
export(resampler)
export(result)
export(result_name)
export(run)
export(seq_in)
export(set_obj_method)
export(set_obj_show)
export(set_struct_obj)
export(struct_class)
export(struct_template)
export(test_metric)
export(value)
exportClasses(struct_class)
exportMethods("$")
exportMethods("$<-")
exportMethods("*")
exportMethods("+")
exportMethods("[")
exportMethods("[<-")
exportMethods("[[")
exportMethods("max_length<-")
exportMethods("models<-")
exportMethods("output_list<-")
exportMethods("output_obj<-")
exportMethods("output_value<-")
exportMethods("param_list<-")
exportMethods("param_obj<-")
exportMethods("param_value<-")
exportMethods("predicted_name<-")
exportMethods("result_name<-")
exportMethods("seq_in<-")
exportMethods("value<-")
exportMethods(.DollarNames)
exportMethods(as.DatasetExperiment)
exportMethods(as.SummarizedExperiment)
exportMethods(as.code)
exportMethods(calculate)
exportMethods(chart_names)
exportMethods(chart_plot)
exportMethods(citations)
exportMethods(evaluate)
exportMethods(export_xlsx)
exportMethods(is_output)
exportMethods(is_param)
exportMethods(length)
exportMethods(libraries)
exportMethods(max_length)
exportMethods(model_apply)
exportMethods(model_predict)
exportMethods(model_reverse)
exportMethods(model_train)
exportMethods(models)
exportMethods(ontology)
exportMethods(output_ids)
exportMethods(output_list)
exportMethods(output_name)
exportMethods(output_obj)
exportMethods(output_value)
exportMethods(param_ids)
exportMethods(param_list)
exportMethods(param_name)
exportMethods(param_obj)
exportMethods(param_value)
exportMethods(predicted)
exportMethods(predicted_name)
exportMethods(result)
exportMethods(result_name)
exportMethods(run)
exportMethods(seq_in)
exportMethods(value)
import(S4Vectors)
import(SummarizedExperiment)
import(datasets)
import(httr2)
import(jsonlite)
import(methods)
importFrom(graphics,plot)
importFrom(knitr,purl)
importFrom(stats,runif)
importFrom(utils,.DollarNames)
importFrom(utils,URLencode)
importFrom(utils,as.person)
importFrom(utils,bibentry)
importFrom(utils,capture.output)
importFrom(utils,citation)
importFrom(utils,file.edit)
